@@ -1,6 +1,7 @@
 import functools
 import re
 import sympy
+import numpy
 import inspect
 
 from .tools import execute_singular_command
@@ -55,6 +56,19 @@ class Ideal(Ideal_Algorithms, object):
         output = execute_singular_command(singular_commands)
         return int(output)
 
+    @property
+    def dims(self):
+        return {entry.count(1) for entry in self.indepSets}
+
+    @property
+    def codim(self):
+        return Ideal(self.ring, ['0']).dim - self.dim
+
+    @property
+    def codims(self):
+        zeroIdeal_dim = Ideal(self.ring, ['0']).dim
+        return {zeroIdeal_dim - dim for dim in self.dims}
+
     @functools.cached_property
     def indepSet(self):
         singular_commands = [f"ring r = {self.ring};",
@@ -75,7 +89,7 @@ class Ideal(Ideal_Algorithms, object):
         output = execute_singular_command(singular_commands)
         if output == 'empty list':
             return []
-        indepSets = [tuple(map(int, line.replace(" ", "").split(","))) for line in output.split("\n") if":" not in line]
+        indepSets = [tuple(map(int, line.replace(" ", "").split(","))) for line in output.split("\n") if ":" not in line]
         return indepSets
 
     @functools.cached_property
@@ -91,6 +105,18 @@ class Ideal(Ideal_Algorithms, object):
         output = [line.replace(",", "") for line in output.split("\n")]
         return output
 
+    @functools.cached_property
+    def leadGBmonomials(self):
+        """Gives the leading monomials of the Groebner basis polynomials."""
+        singular_commands = [f"ring r = {self.ring};",
+                             f"ideal gb = {','.join(self.groebner_basis)};",
+                             "def lts = lead(gb);",
+                             "print(lts);"]
+        output = execute_singular_command(singular_commands)
+        # print(output)
+        output = [line.replace(",", "") for line in output.split("\n")]
+        return output
+
     @functools.cached_property
     def minbase(self):
         singular_commands = [f"ring r = {self.ring};",
@@ -101,6 +127,20 @@ class Ideal(Ideal_Algorithms, object):
         output = [line.replace(",", "") for line in output.split("\n")]
         return output
 
+    @functools.cached_property
+    def radical(self):
+        """Returns the radical of the ideal."""
+        singular_commands = ["LIB \"primdec.lib\";",
+                             f"ring r = {self.ring};",
+                             f"ideal i = {self};",
+                             "def pr = radical(i);",   # options: GTZ / SY
+                             "print(pr);",
+                             "$"]
+        output = execute_singular_command(singular_commands)
+        output = [line.replace(",", "") for line in output.split("\n")]
+        cls, ring = self.__class__, self.ring
+        return cls(ring, output)
+
     @functools.cached_property
     def primary_decomposition(self):
         singular_commands = ["LIB \"primdec.lib\";",
@@ -198,7 +238,7 @@ class Ideal(Ideal_Algorithms, object):
         return cls(ring, output)
 
     def __pow__(self, n):
-        assert(type(n) is int and n >= 0)
+        assert type(n) is int and n >= 0
         cls, ring = self.__class__, self.ring
         if n == 0:
             return cls(ring, ['1'])
@@ -221,6 +261,16 @@ class Ideal(Ideal_Algorithms, object):
         cls, ring = self.__class__, self.ring
         return cls(ring, output)
 
+    @staticmethod
+    def intersection(*args):
+        """Intersection of Ideals - wrapper around & operator for chained intersection."""
+        if len(args) == 1:
+            return args[0]
+        res = args[0] & args[1]
+        for arg in args[2:]:
+            res = res & arg
+        return res
+
     def __truediv__(self, other):
         """Quotient of ideals."""
         singular_commands = [f"ring r = {self.ring};",
@@ -236,14 +286,26 @@ class Ideal(Ideal_Algorithms, object):
 
     def reduce(self, other):
         """Remainder of division, i.e. reduction."""
-        singular_commands = [f"ring r = {self.ring};",
-                             f"ideal gb = {','.join(self.groebner_basis)};",
-                             f"poly f = {other};",
-                             "poly g = reduce(f, gb);",
-                             "print(g);",
-                             "$"]
-        output = execute_singular_command(singular_commands)
-        return output
+        if isinstance(other, Ideal):
+            singular_commands = [f"ring r = {self.ring};",
+                                 f"ideal gb = {','.join(self.groebner_basis)};",
+                                 f"ideal i = {other};",
+                                 "ideal j = reduce(i, gb);",
+                                 "print(j);",
+                                 "$"]
+            output = execute_singular_command(singular_commands)
+            output = output.replace("\n", "").split(",")
+            cls, ring = self.__class__, self.ring
+            return cls(ring, output)
+        else:
+            singular_commands = [f"ring r = {self.ring};",
+                                 f"ideal gb = {','.join(self.groebner_basis)};",
+                                 f"poly f = {other};",
+                                 "poly g = reduce(f, gb);",
+                                 "print(g);",
+                                 "$"]
+            output = execute_singular_command(singular_commands)
+            return output
 
     def __str__(self):
         return ",".join(map(str, self.generators))
@@ -272,3 +334,26 @@ class Ideal(Ideal_Algorithms, object):
 
 def reduce(poly, ideal):
     return ideal.reduce(poly)
+
+
+def monomial_to_exponents(variables, monomial):
+    """Converts a monomial in the variables of a polynomial ring into a numpy.array of exponents."""
+    """Converts a monomial in the variables of a polynomial ring into a numpy.array of exponents."""
+    exps = numpy.zeros(len(variables), dtype=int)
+    variables = list(map(str, variables))  # in case sympy symbols are passsed
+    _split_monomial = list(filter(lambda x: x != '', monomial.replace("^", "**").split("*")))
+    split_monomial = []
+    for entry in _split_monomial:  # need to fix Singular notation for exponents (e.g. if x2 is not a ring variable then x^2 is written as x2)
+        if not entry.isdigit() and entry not in variables:
+            exp = re.findall(r"(\d+)", entry)[0]
+            split_monomial += [entry.replace(exp, ""), exp]
+        else:
+            split_monomial += [entry]
+    for i, ientry in enumerate(split_monomial):
+        if not ientry.isdigit():
+            if i < len(split_monomial) - 1 and split_monomial[i + 1].isdigit():
+                exp = int(split_monomial[i + 1])
+            else:
+                exp = 1
+            exps[variables.index(ientry)] += exp
+    return exps
