@@ -23,6 +23,10 @@ def execute_singular_command(singular_command, timeout=syngular.TIMEOUT, verbose
         raise SingularException(output)
     if '   ? `Q` is undefined' in output:
         output = output.replace("   ? `Q` is undefined", "")
+        print("Singular Error: Q is undefined")
+    if '//options: redSB degBound redefine usage prompt\n' in output:
+        output = output.replace("//options: redSB degBound redefine usage prompt\n", "")
+        print("Singular Warning: redSB degBound redefine usage prompt")
     output = output.replace('// ** minbase applies only to the local or homogeneous case over coefficient fields\n', '')
     output = output.replace('//options: redSB degBound redTail redThrough intStrategy redefine usage prompt\n', '')
     output = output.replace('//options: degBound redTail redThrough intStrategy redefine usage prompt\n', '')
